/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake;

import java.io.IOException;

import krake.queue.Queue;

import org.apache.hadoop.conf.Configured;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.mapreduce.lib.output.SequenceFileOutputFormat;
import org.apache.hadoop.util.Tool;
import org.apache.hadoop.util.ToolRunner;

/**
 * Central job-management class for the krake-crawler. All jobs, including
 * non-hadoop jobs are started through this class.
 * TODO Linking
 * 
 * @author Thomas Nitschke
 */
public class Crawler extends Configured implements Tool
{
	@Override
	public int run(String[] args) throws Exception
	{
		int iters = 1;

		for(String s : args)
		{
			if(s.startsWith("iters="))
			{
				iters = Integer.valueOf(s.substring("iters=".length()));
				if(iters <= 0)
					throw new Exception("Invalid iters count");
			}
			else
				throw new Exception("Unknown command parameter: " + s);
		}
		
		for(int i = 0; i < iters; ++i)
			doCrawl();

		return 0;
	}

	private void doCrawl() throws IOException, InterruptedException, ClassNotFoundException
	{
		getConf().set("mapred.child.java.opts", "-Xmx2000m"); // allow crawlmappers to use more memory
		org.apache.hadoop.mapreduce.Job job = new org.apache.hadoop.mapreduce.Job(getConf(), "Crawler");
		job.setJarByClass(Crawler.class);
		job.setMapperClass(krake.mapreduce.CrawlMapper.class);

		KrakeConfig.load();

		// setup input
		Queue.setupJob(job);

		// setup output
		// set new output directory for this crawl
		Path output_dir = new Path(KrakeConfig.crawl_base + "/" + System.currentTimeMillis()); // TODO create valid path

		// tell the job
		job.setOutputFormatClass(SequenceFileOutputFormat.class);
		SequenceFileOutputFormat.setOutputPath(job, output_dir);
		job.setMapOutputKeyClass(Text.class);
		job.setMapOutputValueClass(Text.class);
		job.setOutputKeyClass(Text.class);
		job.setOutputValueClass(Text.class);

		// no reducer required
		job.setNumReduceTasks(0);
		job.setReducerClass(Reducer.class);
		
		job.submit();
		job.waitForCompletion(true);
		if(!job.isSuccessful())
			throw new IOException("Crawl failed");
	}

	public static void main(String[] args) throws Exception
	{
		int ret = ToolRunner.run(new Crawler(), args);
		System.exit(ret);
	}
}
