/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake;

import krake.mapreduce.IntListWritable;
import org.apache.hadoop.conf.Configured;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.lib.input.SequenceFileInputFormat;
import org.apache.hadoop.mapreduce.lib.output.NullOutputFormat;
import org.apache.hadoop.util.Tool;
import org.apache.hadoop.util.ToolRunner;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class Linker extends Configured implements Tool
{

	public static void main(String[] args) throws Exception
	{
		int ret = ToolRunner.run(new Linker(), args);
		System.exit(ret);
	}

	@Override
	public int run(String[] args) throws Exception
	{
		KrakeConfig.load();

		// init the linker
		Job job = new Job(getConf(), "Linker");
		job.setJarByClass(Linker.class);

		// check the inputfiles
		FileSystem fs = job.getWorkingDirectory().getFileSystem(job.getConfiguration());
		Path id_hash_mapping = new Path(KrakeConfig.id_to_hash_path);

		if(!fs.exists(id_hash_mapping))
			throw new IOException("Inputfiles were not found");

		if(fs.exists(new Path(KrakeConfig.transient_webgraph)))
			throw new IOException("Outputfile already exists");

		// setup input
		job.setInputFormatClass(SequenceFileInputFormat.class);

		// collect all input files (everything in the crawl base directory)
		List<Path> paths = new ArrayList<Path>(32);
		Common.collectInputs(paths, fs, new Path(KrakeConfig.crawl_base));

		for(Path p : paths)
			SequenceFileInputFormat.addInputPath(job, p);

		job.setMapperClass(krake.mapreduce.LinkerMapper.class);
		job.setMapOutputKeyClass(IntWritable.class);
		job.setMapOutputValueClass(IntListWritable.class);
		job.getConfiguration().set("mapred.child.java.opts", "-Xmx6500m"); // allow crawlmappers to use more memory
		
		job.setReducerClass(krake.mapreduce.LinkerReducer.class);

		// linker writes output manually
		job.setOutputFormatClass(NullOutputFormat.class);
		job.setOutputKeyClass(IntWritable.class);
		job.setOutputValueClass(IntListWritable.class);
		job.setNumReduceTasks(1);// limit to one so we have only one output file

		job.submit();
		job.waitForCompletion(true);
		if(!job.isSuccessful())
			throw new Exception("Linker failed");
		return 0;
	}

}
