/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.crawler;


import krake.keys.KeyInfo;

/**
 * Exception that handles errors during a crawl. Holds an errorcode and message for later display.
 * @author Thomas Nitschke
 */
public class CrawlException extends Exception
{
	private static final long serialVersionUID = 7221520045827179876L;

	public CrawlException(KeyInfo ec, String msg)
	{
		super("CrawlErrorCode: " + ec.toString() + "\n" + msg);
		
		message = msg;
		err_code = ec;
	}
	
	public CrawlException(KeyInfo ec)
	{
		super("CrawlErrorCode: " + ec.toString());
		
		message = "";
		err_code = ec;
	}
	
	public KeyInfo getErrorCode() { return err_code; }
	
	/**
	 * @return Unaltered message that was passed to the constructor of the exception.
	 * The normal message may contain additional boilerplate text.
	 */
	public String getVanillaMessage() { return message; }
	
	private KeyInfo err_code;
	private String message;
}
