/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.crawler;

import java.net.URL;

/**
 * Cache which stores the last seen URLs. The cache uses a simplified LRU-system and removes the URL
 * that was added first to the cache.
 * @author Thomas Nitschke
 */
public class URLCache
{
	private static final int SIZE = 64;

	/**
	 * Checks if the URL is in the cache and inserts it if its missing.
	 * This may remove the oldest URL from the cache.
	 * @param u URL
	 * @return true if the URL is in the cache, false otherwise.
	 */
	public boolean query(URL u)
	{
		for(int i = 0; i < SIZE; ++i)
		{
			if(m_cache[i] == null)
				continue;
			else if(m_cache[i].toString().equals(u.toString()))
				return true;
		}

		// fill null-places first
		if(m_inserts < SIZE)
		{
			for(int i = 0; i < SIZE; ++i)
			{
				if(m_cache[i] == null)
				{
					m_cache[i] = u;
					break;
				}
			}
		} else
			// overwrite
			m_cache[m_inserts % SIZE] = u;

		++m_inserts;
		return false;
	}

	private URL[] m_cache = new URL[SIZE];
	private int m_inserts = 0;
}
