/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.file;

import java.io.IOException;
import java.util.zip.DataFormatException;
import java.util.zip.Inflater;

/**
 * Base for all classes that work with the krake content file.
 * @author Thomas Nitschke
 *
 */
public abstract class Content
{

	/**
	 * Decompresses the supplied byte-buffer (entry in the content file) to a java string.
	 * @param zipped Compressed data
	 * @return String of the decompressed data
	 * @throws IOException
	 */
	protected String uncompress(byte[] zipped) throws IOException
	{
		Inflater decompresser = new Inflater();
		decompresser.setInput(zipped, 0, zipped.length);
		
		byte[] result = new byte[zipped.length*10];
		int size;
		try
		{
			size = decompresser.inflate(result);
		}
		catch(DataFormatException e)
		{
			e.printStackTrace();
			throw new IOException("Corrupted compressed data\n" + e.getMessage());
		}
		decompresser.end();

		// decode the bytes into a string
		return new String(result, 0, size, "UTF-8");
	}
}
