/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.file;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;

import java.io.IOException;
import java.io.InputStream;

/**
 * Provides streaming access to a krake content file in the hadoop DFS.
 * @author Thomas Nitschke
 */
public class ContentDFSStream extends ContentStream
{
	private FSDataInputStream m_in = null;

	/**
	 * Opens the content file.
	 * @param p DFS path to the content file
	 * @throws IOException
	 */
	public ContentDFSStream(Path p) throws IOException
	{
		Configuration config = new Configuration();
		FileSystem dfs = FileSystem.get(config);
		m_in = dfs.open(p);
	}

	@Override
	protected InputStream getIn()
	{
		return m_in;
	}

	@Override
	public void close()
	{
		try
		{
			m_in.close();
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
		finally
		{
			m_in = null;
		}
	}

}
