/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.file;

import krake.Common;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.*;


/**
 * Provides random access to a local krake content file.
 * @author Thomas Nitschke
 */
public class ContentLocalRandomAccess extends ContentRandomAccess
{
	private static final Log LOG = LogFactory.getLog(ContentLocalRandomAccess.class);
	private RandomAccessFile m_in;

	/**
	 * Reads the offsetfile and opens the contentfile for random access.
	 * The offsetfile is loaded into memory!
	 * @param offsetfile Offsetfile to read
	 * @param contentfile Contentfile to open
	 * @throws Exception
	 */
	public ContentLocalRandomAccess(File offsetfile, File contentfile) throws Exception
	{
		BufferedInputStream offstream = new BufferedInputStream(new FileInputStream(offsetfile));
		readOffsetTable(offstream, offsetfile.length());
		offstream.close();
		
		m_in = new RandomAccessFile(contentfile, "r");

		LOG.info("Verifying");
		verify();
		m_ids = null;
	}

	@Override
	public void close()
	{
		try
		{
			m_in.close();
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
	}
	
	private int readSize(RandomAccessFile in) throws IOException
	{
		byte[] integer = new byte[4];
		readBuffer(in, integer, integer.length);
		return Common.intFromBytes(integer);
	}

	private void readBuffer(RandomAccessFile in, byte[] buffer, int to_read) throws IOException
	{
		int off = 0;
		int missing = to_read;
		int read;

		while(missing > 0)
		{
			read = in.read(buffer, off, missing);
			if(read == -1)
				throw new EOFException();

			off += read;
			missing -= read;
		}
	}

	@Override
	public String get(int id) throws IOException
	{
		if(id >= getEntryCount())
			return null;
		
		long offset = m_offsets[id];
		
		m_in.seek(offset);
		int read_size = readSize(m_in);
		
		if(read_size == 0)
			return "";
		
		byte[] buffer = new byte[read_size];
		readBuffer(m_in, buffer, buffer.length);
		
		return uncompress(buffer);
	}
	
	@Override
	public String get_next() throws IOException
	{
		// read the size of the last node
		readSize(m_in);
		
		// read the size of the new node
		int read_size = readSize(m_in);
		if(read_size == 0)
			return "";
		
		byte[] buffer = new byte[read_size];
		readBuffer(m_in, buffer, buffer.length);
		
		return uncompress(buffer);
	}
}
