/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.file;

import java.io.*;

/**
 * Provides streaming access to a local krake content file.
 * @author Thomas Nitschke
 */
public class ContentLocalStream extends ContentStream
{
	private BufferedInputStream m_in = null;

	/**
	 * Open a local krake content file.
	 * @param file File to open
	 * @throws IOException
	 */
	public ContentLocalStream(File file) throws IOException
	{
		m_in = new BufferedInputStream(new FileInputStream(file));
		m_cur_size = readSize(m_in);
		if(m_cur_size <= 0)
			throw new IOException("Invalid content file, size is <= 0");
	}

	@Override
	public void close()
	{
		try
		{
			m_in.close();
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
		finally
		{
			m_in = null;
		}
	}

	@Override
	protected InputStream getIn()
	{
		return m_in;
	}
}
