/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.file.unused;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.io.*;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;

public class DomainToIdsMapping
{
	private final static Log LOG = LogFactory.getLog(DomainToIdsMapping.class);
	
	public static DomainToIdsMapping loadLocal(String path) throws IOException
	{
		DataInputStream in = new DataInputStream(new FileInputStream(path));
		DomainToIdsMapping instance = new DomainToIdsMapping(in);
		in.close();
		return instance;
	}
	
	public static DomainToIdsMapping loadDFS(String path) throws IOException
	{
		throw new IOException("Not implemented yet");
	}
	
	
	private HashMap<String, int[]> mapping;
	
	private DomainToIdsMapping(DataInputStream in) throws IOException
	{
		mapping = new HashMap<String, int[]>(1000000);
		
		// parse the csv-format (tab separated)
		BufferedReader r = new BufferedReader(new InputStreamReader(in));

		LOG.info("Reading domain to nodes mapping...");
		while(true)
		{
			String line = r.readLine();
			if(line == null)
				break;
			
			String[] segments = line.split("\t");
			
			int[] nodes = new int[segments.length - 1];
			boolean format_error = false;
			
			for(int i = 1; i < segments.length; ++i)
			{
				try
				{
					nodes[i-1] = Integer.parseInt(segments[i]);
				}
				catch(NumberFormatException e)
				{
					LOG.warn("Skipping unparseble line " + line);
					format_error = true;
					break;
				}
			}
			
			// skip invalid lines
			if(format_error)
				continue;
			
			mapping.put(segments[0], nodes);
		}
		
		LOG.info("done");
	}
	
	
	public final int[] getNodes(String domain)
	{
		return mapping.get(domain);
	}
	
	
	public final Iterator<Entry<String, int[]>> getIterator()
	{
		return mapping.entrySet().iterator();
	}
	
	public final int getDomainCount()
	{
		return mapping.size();
	}
	
	public final void getDomains(String[] dest)
	{
		if(dest.length != getDomainCount())
			throw new IllegalArgumentException("The size of the string-array must be equal to the domain count");
		mapping.keySet().toArray(dest);
	}
}
