package krake.file.unused;

import krake.Common;
import krake.KrakeConfig;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;

import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;

/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
public class IdToURLMapping
{
	private final static Log LOG = LogFactory.getLog(IdToURLMapping.class);
	private int[] ranges = null;
	private byte[] data = null;
	
	
	public static IdToURLMapping openDFS(FileSystem fs, Path p) throws IOException
	{
		FSDataInputStream in = fs.open(p);
		IdToURLMapping mapping = new IdToURLMapping(in);
		in.close();
		return mapping;
	}
	
	
	public static IdToURLMapping openLocal(String path) throws IOException
	{
		DataInputStream in = new DataInputStream(new FileInputStream(path));
		IdToURLMapping mapping = new IdToURLMapping(in);
		in.close();
		return mapping;
	}
	
	private IdToURLMapping(DataInputStream in) throws IOException
	{
		int count = in.readInt();
		LOG.info(count + " elements in mapping");
		
		ranges = new int[count * 2];
		for(int i = 0; i < count * 2; ++i)
			ranges[i] = in.readInt();
		
		
		int datasize = ranges[count * 2 - 2] + ranges[count * 2 - 1];
		LOG.info("Reading " + datasize + " bytes of data");
		
		data = new byte[datasize];
		in.readFully(data);
	}
	
	public int getSize()
	{
		return ranges.length / 2;
	}
	
	public String get(int id)
	{
		int offset = ranges[id * 2];
		int length = ranges[id * 2 + 1];
		return new String(data, offset, length);
	}
	
	public static void main(String[] args) throws IOException
	{
		Path src_path = null;
		
		printAllURLs(src_path);
	}
	
	private static void printAllURLs(Path src_path) throws IOException
	{
		DataInputStream in = new DataInputStream(new FileInputStream("/home/cc/id_to_url.bin"));
		IdToURLMapping mapping = new IdToURLMapping(in);
		in.close();
		
		for(int i = 0; i < mapping.getSize(); ++i)
		{
			String url = mapping.get(i);
			URL rurl = new URL(url);
			if(rurl.getHost().equals("web1.156061.vserver.de"))
				System.out.println(rurl.getPath() + rurl.getQuery());
		}
		
		FileOutputStream out = new FileOutputStream("/home/cc/urls2.txt");
		
		
		HashMap<String, ArrayList<Integer>> dists = new HashMap<String, ArrayList<Integer>>(500000);
		
		out.write("Id\tLength\tDomain\n".getBytes());
		for(int i = 0; i < mapping.getSize(); ++i)
		{
			String url = mapping.get(i);
			URL rurl = new URL(url);
			int length = rurl.getPath().length();
			out.write((i + "\t" + length + "\t" + rurl.getHost() + "\n").getBytes());
			
			ArrayList<Integer> ints = dists.get(rurl.getHost());
			if(ints == null)
			{
				ints = new ArrayList<Integer>(100);
				dists.put(rurl.getHost(), ints);
			}
			
			ints.add(length);
		}
		out.close();
		
		out = new FileOutputStream("/home/cc/urls_dist.txt");
		
		for(Iterator<Entry<String, ArrayList<Integer>>> it = dists.entrySet().iterator(); it.hasNext();)
		{
			Entry<String, ArrayList<Integer>> e = it.next();
			if(e.getValue().size() <= 90 || e.getValue().size() > 110)
				continue;
			
			Collections.sort(e.getValue());
			
			out.write((e.getKey() + "\t").getBytes());
			for(int i : e.getValue())
				out.write((i + "\t").getBytes());
			out.write("\n".getBytes());
		}
		out.close();
	}

	@SuppressWarnings("unused")
	private static void checkMappingConsistency(Path src_path, Path dst_path) throws IOException, MalformedURLException
	{
		KrakeConfig.load();
		Configuration config = Common.getDefaultHadoopConfig();
		FileSystem fs = FileSystem.get(config);
		
		FSDataInputStream in = fs.open(src_path);
		IdToURLMapping mapping = new IdToURLMapping(in);
		in.close();
		
		//URLMapping idToHash = new URLMapping(Opener.openMappingDFS(fs, new Path("totalexport/id_to_hash.fbin")));
		
		BufferedWriter out = new BufferedWriter(new FileWriter(new File("./1000urls")));
		
		for(int i = 0; i < 5000; ++i)
		{
			String surl = mapping.get(i);
			URL url = new URL(surl);
			
			out.write(i + " " + url + "\n");
			
//			String hash = Common.normalize(url);
			
//			int mapped_id = idToHash.hash2int(Common.HextoByteArray(hash));
//			if(mapped_id != i)
//			{
//				System.out.println("Invalid mapping between '" + surl + "' " + surl.length() + "\nas " + hash + "\n" + mapped_id + " <-> " + i);
//				throw new IOException();
//			}
		}
		
		out.close();
		System.out.println("Done");
	}
}
