/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.graph;

/**
 * Interface for all web graph related classes
 * @author Thomas Nitschke
 */
public interface IWebGraph
{
	/**
	 * Gives access to the inlinks of a given node.
	 * @param id ID of the node
	 * @return array of nodes with links to the given node 
	 */
	public int[] getInlinks(int id);
	
	/**
	 * Gives access to the outlinks of a given node.
	 * @param id ID of the node
	 * @return array of nodes with links from the given node 
	 */
	public int[] getOutlinks(int id);
	
	/**
	 * Gives access to all links (in-/outlinks) of a given node.
	 * @param id ID of the node
	 * @return array of nodes with links to and from the given node 
	 */
	public int[] getAlllinks(int id);
	
	
	/**
	 * Provides access to the number of nodes of the graph.
	 * @return number of nodes
	 */
	public int getNumberOfNodes();

	/**
	 * Checks whether there exists an outgoing edge from node i to node j where
	 * i and j are given by their hash values.
	 *
	 * @param hash1
	 *            source node
	 * @param hash2
	 *            target node
	 * @return <b>true</b> iff thequeried edge exists
	 */
	boolean isEdge(int i, int j);

   	/**
	 * Getter for the number of edges contained in the graph.
	 *
	 * @return number of edges of the graph
	 */
	long getNumberOfEdges();

	/**
	 * Computes the averall degree (indegree + outdegree) of a given node.
	 *
	 * @param inode
	 *            a node for which to compute the degree
	 * @return degree of the given node
	 */
	int getDegree(int inode);

	/**
	 * Comutes the indegree of a given node.
	 *
	 * @param inode
	 *            node ID for which to compute the indegree
	 * @return indegree of the given node
	 */
	int getIndegree(int inode);

 	/**
	 * Comutes the outdegree of a given node.
	 *
	 * @param inode
	 *            node ID for which to compute the outdegree
	 * @return outdegree of the given node
	 */
	int getOutdegree(int inode);
}
