/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.mapreduce;

import krake.crawler.KrakeCrawler;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Mapper;

import java.io.IOException;

/**
 * Mapper-Part for the Crawling-Crawler, reads URLs with Hash from the queue and
 * sends them to the crawler.
 * Emits crawled pages.
 * @author Thomas Nitschke
 */
public class CrawlMapper extends Mapper<Text, Text, Text, Text>
{
	private final static Log LOG = LogFactory.getLog(CrawlMapper.class);
	private KrakeCrawler m_crawler = null;

	public void setup(Context ctx)
	{
		// initialize the crawler
		try
		{
			m_crawler = new KrakeCrawler();
			m_crawler.init(ctx);
		}
		catch(IOException e)
		{
			e.printStackTrace();
			LOG.error("Failed to create crawler" , e);

			m_crawler = null;
		}
	}

	public void cleanup(Context ctx)
	{
		m_crawler.shutDown();
	}

	public void map(Text ihash, Text iurl, Context context) throws IOException, InterruptedException
	{
		// handle errors that occured during setup
		if(m_crawler == null)
			throw new IOException("Crawler setup failed");

		// send input to the crawler
		String hash = ihash.toString();
		String url = iurl.toString();
		m_crawler.processURL(hash, url);
	}
}
