/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.mapreduce;

import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;

import krake.Common;
import krake.KrakeConfig;

import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Counter;
import org.apache.hadoop.mapreduce.Reducer;

/**
 *
 * @author Thomas Nitschke
 */
public class ExportCounterReducer extends Reducer<Text, TextPairWritable, IntWritable, IntWritable>
{
	private HashSet<String> unique_urls;
	private Counter ctr_pages;

	@Override
	protected void setup(Context ctx)
	{
		ctr_pages = ctx.getCounter(Common.CrawlCounters.PAGES);

		int hashsize = 2000;
		try
		{
			KrakeConfig.load();
			hashsize = KrakeConfig.batch_size * 2;
		}
		catch(Exception e)
		{
		}
		unique_urls = new HashSet<String>(hashsize);
	}

	@Override
	protected void cleanup(Context ctx)
	{
		unique_urls.clear();
	}

	@Override
	protected void reduce(Text hash, Iterable<TextPairWritable> pages, Context ctx) throws IOException, InterruptedException
	{
		Iterator<TextPairWritable> it = pages.iterator();
		if(it.hasNext())
			ctr_pages.increment(1);
		else
			throw new IOException("Key without values in reducer");
	}
}
