/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.mapreduce;

import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.Writable;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * Writable which contains a list of integers.
 * @author Thomas Nitschke
 */
public class IntListWritable implements Writable
{
	private List<IntWritable> data = null;
	
	public IntListWritable()
	{
		data = new ArrayList<IntWritable>(1);
	}
	
	public IntListWritable(List<IntWritable> ints)
	{
		data = new ArrayList<IntWritable>(ints.size());
		data.addAll(ints);
	}

	public IntListWritable(Set<IntWritable> ints)
	{
		data = new ArrayList<IntWritable>(ints.size());
		data.addAll(ints);
	}
	
	public IntListWritable(IntWritable[] array)
	{
		data = new ArrayList<IntWritable>(array.length);
		for(IntWritable i : array)
			data.add(i);
	}
	
	public void add(IntWritable i)
	{
		data.add(i);
	}
	
	@Override
	public void readFields(DataInput input) throws IOException
	{
		int size = input.readInt();
		
		data = new ArrayList<IntWritable>(size);
		for(int i = 0; i < size; ++i)
			data.add(new IntWritable(input.readInt()));

	}

	@Override
	public void write(DataOutput output) throws IOException
	{
		output.writeInt(data.size());
		
		for(IntWritable i : data)
			output.writeInt(i.get());
	}
	
	public final List<IntWritable> getInts() { return data; }
	
	public String toString()
	{
		StringBuilder builder = new StringBuilder(128);
		
		for(IntWritable i : data)
		{
			builder.append(i.get());
			builder.append('\t');
		}
		return builder.toString();
	}

}
