/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.mapreduce;

import krake.KrakeConfig;
import org.apache.hadoop.fs.FSDataOutputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.mapreduce.Reducer;

import java.io.IOException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

public class LinkerReducer extends Reducer<IntWritable, IntListWritable, IntWritable, IntListWritable>
{
	private FSDataOutputStream out;
	private Exception setup_error = null;
	private byte[] tab;
	private byte[] nl;

	@Override
	public void setup(Context ctx)
	{
		try
		{
			tab = "\t".getBytes("UTF-8");
			nl = "\n".getBytes("UTF-8");

			KrakeConfig.load();
			Path path = new Path(KrakeConfig.transient_webgraph);
			FileSystem fs = FileSystem.get(ctx.getConfiguration());

			out = fs.create(path, true);
		}
		catch(Exception e)
		{
			setup_error = e;
		}
	}

	@Override
	public void cleanup(Context ctx)
	{
		try
		{
			out.close();
		}
		catch(IOException e)
		{
		}
	}

	@Override
	public void reduce(IntWritable node, Iterable<IntListWritable> listolist, Context ctx) throws IOException, InterruptedException
	{
		if(setup_error != null)
			throw new IOException(setup_error);

		Iterator<IntListWritable> ilistolist = listolist.iterator();
		Set<IntWritable> links = new HashSet<IntWritable>(128);

		while(ilistolist.hasNext())
		{
			List<IntWritable> l = ilistolist.next().getInts();
			if(l == null)
				continue;
			
			for(IntWritable link : l)
				links.add(link);
			
			l = null;
		}

		out.write(node.toString().getBytes("UTF-8"));
		for(IntWritable i : links)
		{
			out.write(tab);
			out.write(i.toString().getBytes("UTF-8"));
		}
		out.write(nl);
	}
}
