/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.mapreduce.formats;

import org.apache.hadoop.conf.Configurable;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.mapreduce.InputSplit;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

/**
 * Inputsplit for ContentFileInputFormat. See hadoop documentation about input formats and splits
 * for further details.
 * @author Thomas Nitschke
 */
public class ContentFileInputSplit extends InputSplit implements Writable, Configurable
{
	private static final long serialVersionUID = -5350617853958166374L;
	private int m_count;
	private int m_first;
	private Configuration m_conf;
	
	public ContentFileInputSplit(int count, int first)
	{
		m_count = count;
		m_first = first;
	}
	
	public ContentFileInputSplit() { }

	@Override
	public void readFields(DataInput in) throws IOException
	{
		m_count = in.readInt();
		m_first = in.readInt();
	}

	@Override
	public void write(DataOutput out) throws IOException
	{
		out.writeInt(m_count);
		out.writeInt(m_first);
	}
	
	int getFirst() { return m_first; }
	int getCount() { return m_count; }

	@Override
	public long getLength() throws IOException, InterruptedException { return (long)m_count; }

	@Override
	public String[] getLocations() throws IOException, InterruptedException { return new String[0]; }

	@Override
	public Configuration getConf() { return m_conf; }

	@Override
	public void setConf(Configuration conf) { m_conf = conf; }
}