/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.mapreduce.formats;

import org.apache.hadoop.io.Writable;
import org.apache.hadoop.mapreduce.InputSplit;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

/**
 * InputSplit for the local queue backend. Each split has just one location (server) to which it belongs.
 * The input format creates these splits in order to spawn a mapreduce process on every node.
 */
public class QueueLocalInputSplit extends InputSplit implements Writable
{
	private String[] home = new String[]{""};

	public QueueLocalInputSplit(String home_server)
	{
		assert(home_server != null);
		home[0] = home_server;
	}

	public QueueLocalInputSplit()
	{
	}


	/**
	 * TODO return the batchsize
	 * Always returns a constant since the size of the split cannot be measured.
	 * @return
	 * @throws IOException
	 * @throws InterruptedException
	 */
	@Override
	public long getLength() throws IOException, InterruptedException
	{
		return Long.MAX_VALUE / 2;
	}

	@Override
	public String[] getLocations() throws IOException, InterruptedException
	{
		return home;
	}

	@Override
	public void write(DataOutput dataOutput) throws IOException
	{
		dataOutput.writeUTF(home[0]);
	}

	@Override
	public void readFields(DataInput dataInput) throws IOException
	{
		String tmp = dataInput.readUTF();
		home[0] = tmp;
	}

	// part of the writable interface
	public static QueueLocalInputSplit read(DataInput in) throws IOException
	{
		QueueLocalInputSplit qs = new QueueLocalInputSplit();
		qs.readFields(in);
		return qs;
	}
}
