/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.search;

import krake.Common;
import krake.KrakeConfig;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FSDataOutputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;

import java.io.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.StringTokenizer;

/**
 * Reads the output of the InvertedIndexJob and writes a condensed binary index file and a text header file
 * which adresses index-sets in the index file. The result of the InvertexIndexFinalizer is used by the SearchServer as database.
 * @author Thomas Nitschke
 *
 */
public class InvertedIndexFinalizer
{
	public static void main(String[] args) throws IOException
	{
		ArrayList<Integer> nodes = new ArrayList<Integer>(100000);
		
		KrakeConfig.load();
		Configuration config = Common.getDefaultHadoopConfig();
		FileSystem fs = FileSystem.get(config);
		FSDataOutputStream out = fs.create(new Path("invertedindex/invidx_nodes.bin"));
		FSDataOutputStream header = fs.create(new Path("invertedindex/invidx_header.txt"));
		BufferedWriter w = new BufferedWriter(new OutputStreamWriter(header));
		
		FSDataInputStream in = fs.open(new Path("invertedindex/part-r-00000"));
		BufferedReader r = new BufferedReader(new InputStreamReader(in));
		
		/*
		 * Write the node ids as binary integer and record the start offset
		 * and size of each set in the index.
		 */
		while(true)
		{
			// parse line
			String line = r.readLine();
			if(line == null)
				break;
			StringTokenizer tokenizer = new StringTokenizer(line);
			line = null;
			
			// read word
			String word = tokenizer.nextToken();
			long offset = out.getPos();
			
			// read nodes
			while(tokenizer.hasMoreTokens())
			{
				int node = Integer.parseInt(tokenizer.nextToken());
				nodes.add(node);
			}
			
			// write sorted node list
			Collections.sort(nodes);
			for(int i : nodes)
				out.writeInt(i);
			
			
			// write header table entry
			w.write(word + "\t" + nodes.size() + "\t" + offset + "\n");
			nodes.clear();
		}
		
		in.close();
		out.close();
		w.close();
		header.close();
	}
}
