/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.search;

import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Mapper;

import java.io.IOException;
import java.util.HashSet;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Mapper of the InvertedIndexJob. Parses each given website and extracts its text.
 * Words which contain non-alphanumeric characters or have less than 4 or more than
 * 32 characters are omited from the index.
 * @author Thomas Nitschke
 *
 */
public class InvertedIndexMapper extends Mapper<IntWritable, Text, Text, IntWritable>
{
	private HashSet<String> set = new HashSet<String>(1000);
	private Pattern extract_words_pattern = Pattern.compile("(<[^<>]*>)|(\\\\n)|(\\\\t)");
	private Pattern natural_word_pattern = Pattern.compile("\\w+");
	
	public void map(IntWritable key, Text value, Context ctx) throws IOException, InterruptedException
	{
		Matcher matcher = extract_words_pattern.matcher(value.toString()); 
		String pureText = matcher.replaceAll(" ");
		
		// iterate over words and unify with a set container
		StringTokenizer tokenizer = new StringTokenizer(pureText);
		while(tokenizer.hasMoreTokens())
		{
			String token = tokenizer.nextToken();
			token = token.trim().toLowerCase();
			
			// ignore small/large words
			if(token.length() <= 3 || token.length() > 32)
				continue;
			
			// ignore strange non-natural words
			matcher = natural_word_pattern.matcher(token);
			if(!matcher.matches())
				continue;
			
			set.add(token);
		}
		
		// write out
		for(String s : set)
			ctx.write(new Text(s), key);
		
		set.clear();
	}
}