/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.search;
/*
import krake.Common;
import krake.KrakeConfig;
import krake.Pair;
import krake.file.ContentDFSRandomAccess;
import krake.file.unused.IdToURLMapping;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.simpleframework.http.Form;
import org.simpleframework.http.Request;
import org.simpleframework.http.Response;
import org.simpleframework.http.core.Container;
import org.simpleframework.transport.connect.Connection;
import org.simpleframework.transport.connect.SocketConnection;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
*/

/**
 * Minimalistic HTTP-Server which acts as frontend for the inverted index and the feature matrix.
 * The server answers search queries with two result sets which are ordered by the page rank and the thermo rank.
 * @author Thomas Nitschke
 * This code requires the simpleframework (a simple http server implementation) amd therefore is commented out.
 */
class SearchServer
{
	// dummy, use class below
}
/*
public class SearchServer implements Container
{
	private static final Log LOG = LogFactory.getLog(SearchServer.class);
	private IdToURLMapping urls;
	private Map<String, Pair<Integer, Long>> header = new HashMap<String, Pair<Integer, Long>>(5000000);
	private float[] rank_page;
	private float[] rank_thermo;
	private ContentDFSRandomAccess content;
	private FSDataInputStream invidx;
	private FileSystem fs;
	private Path path_thermorank;
	private Path path_pagerank;

	private int[] IntersecLin(int[] a, int[] b)
	{
		// a is the bigger list
		if(a.length < b.length)
		{
			int[] c = a;
			a = b;
			b = c;
		}

		int wj = 0;
		int j = 0;
		for(int i = 0; i < b.length;)
		{
			if(i == b.length || j == a.length)
				break;

			if(a[j] > b[i])
				++i;
			else if(a[j] < b[i])
				++j;
			else
			{
				b[wj] = a[j];
				++wj;
				++i;
				++j;
			}
		}

		return Arrays.copyOf(b, wj);
	}

	private int[] IntersecSubLin(int[] a, int[] b)
	{
		// a is the bigger list
		if(a.length < b.length)
		{
			int[] c = a;
			a = b;
			b = c;
		}

		int log = 31 - Integer.numberOfLeadingZeros(a.length);
		if(log * b.length > a.length + b.length)
			return IntersecLin(a, b);

		int wj = 0;
		for(int i = 0; i < b.length; ++i)
		{
			if(Arrays.binarySearch(a, b[i]) >= 0)
			{
				b[wj] = b[i];
				++wj;
			}
		}

		return Arrays.copyOf(b, wj);
	}

	private int[] queryInvertedIndex(String[] keys)
	{
		// limit amount of keys and check for empty query
		if(keys.length == 0 || keys.length > 32)
			throw new IllegalArgumentException("Query is empty of too big");

		int[] result = getIndicesForKey(keys[0]);

		// no intersection to compute, or empty set in intersection
		if(keys.length == 1 || result.length == 0)
			return result;

		// intersect the sets one by one (better: smallest first)
		int[] other;
		for(int i = 1; i < keys.length; ++i)
		{
			// get the next "other" set
			other = getIndicesForKey(keys[i]);
			if(other.length == 0)
				return other;
			
			LOG.info("Intersecting sets: " + result.length + "/" + other.length);
			result = IntersecSubLin(result, other);
			LOG.info("Result set after intersection: " + result.length + " elements");

			// empty set reached
			if(result.length == 0)
				return result;
		}
		
		return result;
	}
	
	private void loadHeader(Path p) throws IOException
	{
		LOG.info("Loading header for inverted index...");
		FSDataInputStream in = fs.open(p);
		BufferedReader r = new BufferedReader(new InputStreamReader(in));
		
		int counter = 0;
		while(true)
		{
			String line = r.readLine();
			if(line == null)
				break;
			
			String[] parts = line.split("\\s");
			if(parts.length != 3)
				throw new IOException("Invalid line in header");
			
			int count = Integer.parseInt(parts[1]);
			long offset = Long.parseLong(parts[2]);
			
			if(count < 3)
				continue;
			
			header.put(parts[0], new Pair<Integer, Long>(count, offset));
			
			if(counter % 10000 == 0)
				LOG.info("Loaded " + counter + " entries");
			++counter;
		}
		
		in.close();
		LOG.info("done");
	}
	
	public SearchServer(
			FileSystem fs, 
			Path content, 
			Path offsets, 
			Path invertedindex,
			Path indexheader, 
			Path id2url, 
			Path pagerank, 
			Path thermorank) throws IOException
	{
		LOG.info("Loadind data for search server...");
		this.fs = fs;
		path_pagerank = pagerank; 
		path_thermorank = thermorank;
		
		urls = IdToURLMapping.openDFS(fs, id2url);
		loadHeader(indexheader);
		this.content = new ContentDFSRandomAccess(fs.getConf(), offsets, content);
		invidx = fs.open(invertedindex);
		loadRanks();
		LOG.info("Loading complete");
	}
	
	private void loadRanks() throws IOException
	{
		LOG.info("Loading ranks");
		double[] tmp;
		
		FSDataInputStream in = fs.open(path_pagerank);
		tmp = Rank.loadRankArray(in);
		in.close();
		
		rank_page = new float[tmp.length];
		for(int i = 0; i < tmp.length; ++i)
			rank_page[i] = (float)tmp[i];
		
		
		in = fs.open(path_thermorank);
		tmp = Rank.loadRankArray(in);
		if(tmp.length != rank_page.length)
			throw new IOException("Rankfiles have different length");
		in.close();
		
		rank_thermo = new float[tmp.length];
		for(int i = 0; i < tmp.length; ++i)
			rank_thermo[i] = (float)tmp[i];
		LOG.info("done");
	}
	
	public void finalize()
	{
		try
		{
			invidx.close();
			content.close();
		}
		catch(IOException e)
		{
		}
		
		try
		{
			super.finalize();
		}
		catch(Throwable e)
		{
			e.printStackTrace();
		}
	}

	private int[] getIndicesForKey(String key)
	{
		LOG.info("Getting indices for key " + key);
		
		Pair<Integer, Long> entry = header.get(key);
		if(entry == null)
			return new int[]{};
		
		try
		{
			invidx.seek(entry.m_second);
			
			int[] nodes = new int[entry.m_first];
			for(int i = 0; i < nodes.length; ++i)
				nodes[i] = invidx.readInt();
			
			return nodes;
		}
		catch(IOException e)
		{
			LOG.error("Unable to get indices for key", e);
			return new int[]{};
		}
	}
	
	private class RankComparator implements Comparator<Integer>
	{
		private float[] ranks;
		
		public RankComparator(float[] ranks)
		{
			this.ranks = ranks;
		}

		@Override
		public int compare(Integer o1, Integer o2)
		{
			return Float.compare(ranks[o2], ranks[o1]);
		}
	}
	
	private Integer[] sortIndices(int[] idx, float[] ranks)
	{
		Integer[] oidx = new Integer[idx.length];
		for(int i = 0; i < idx.length; ++i)
			oidx[i] = new Integer(idx[i]);
		
		Arrays.sort(oidx, new RankComparator(ranks));
		
		return oidx;
	}

	public static void main(String[] args) throws IOException
	{
		LOG.info("Starting search server");
		KrakeConfig.load();
		Configuration config = Common.getDefaultHadoopConfig();
		FileSystem fs = FileSystem.get(config);
		
		Container container = new SearchServer(fs,
				new Path("totalexport/content.bin"),
				new Path("totalexport/offsets.bin"),
				new Path("invertedindex/invidx_nodes.bin"),
				new Path("invertedindex/invidx_header.txt"),
				new Path("totalexport/id_to_url.bin"),
				new Path("totalexport/page.prk"),
				new Path("totalexport/thermo.prk"));
		Connection connection = new SocketConnection(container);
		SocketAddress address = new InetSocketAddress(8080);
		LOG.info("Listening");

		connection.connect(address);

	}
	
	private static final String page_start = "<html> <title>LS2 Suche</title> <body><div style=\"margin: 0px auto; margin-top: 300px; width:1000px; text-align:center;\"> <h1>LS2 Suche</h1> <form action=\"search\" method=\"GET\"> <input name=\"query\" type=\"text\" size=\"48\" maxlength=\"128\" /> </form> <a style='margin-top: 200px' href='/reload'>Reload ranks</a></div> </body> </html>"; 
	private static final String page_search_header = "<html><title>LS2 Suche</title><body>";
	private static final String page_search_footer = "</body></html>";
	
	@Override
	public void handle(Request request, Response response)
	{
		PrintStream body;
		try
		{
			long time_start = System.currentTimeMillis();
			LOG.info("Got request " + request.getPath() + request.getQuery());
			
			body = response.getPrintStream();
			response.set("Content-Type", "text/html");
			response.set("Server", "LS2 HyperMegaGlobalP!=NP Server/1.0");
			response.setDate("Date", time_start);
			response.setDate("Last-Modified", time_start);
			
			String path = request.getPath().toString();
			
			if(path.equals("/search"))
			{
				Form form = request.getForm();
				if(form == null)
					printStartPage(body);
				else
				{
					String keystring = form.get("query");
					if(keystring == null)
						printStartPage(body);
					else
						handleSearchRequest(body, keystring);
				}
			}
			else if(path.equals("/reload"))
				handleRankReload(body);
			else
			{
				try
				{
					handlePageRequest(body, path);
				}
				catch(Exception e)
				{
					printStartPage(body);
				}
			}
			
			body.close();
			LOG.info("Processed request " + request.getPath() + request.getQuery() + " in " + (System.currentTimeMillis() - time_start) + " ms");
		}
		catch(IOException e)
		{
			LOG.error(e.getMessage(), e);
			e.printStackTrace();
		}
	}
	
	private void handleRankReload(PrintStream body) throws IOException
	{
		body.println("<html><body><h3>Reloading ranks, this may take a minute or two.<br />");
		body.println("<a href='/'>Return to search</a></h3></body></html>");
		loadRanks();
	}

	private void handlePageRequest(PrintStream body, String path) throws IOException
	{
		int node = Integer.parseInt(path.substring(1));
		String source = content.get(node);
		body.print(source);
	}

	private void handleSearchRequest(PrintStream body, String keystring)
	{
		keystring = keystring.toLowerCase();
		System.out.println(keystring);
		String[] keys = keystring.split("\\s");
		
		
		int[] nodes = queryInvertedIndex(keys);
		
		body.println(page_search_header);
		if(nodes.length > 0)
		{
			int limit = Math.min(nodes.length, 500);
			
			Integer[] idx = sortIndices(nodes, rank_page);
			body.println("<div style='float: left; width: 45%; margin-left: 10px;'>");
			for(int i = 0; i < limit; ++i)
			{
				int node = idx[i];
				
				body.println("<div style='overflow: hidden; margin: 0px auto; width:90%; margin-bottom: 20pt; border-color: grey; border-style: dashed; border-width: 1px; padding: 5px;'>");
				body.println("<div style='font-size:14px; '>");
				body.println("<a href='/" + node + "'>" + urls.get(node) + "</a>");
				body.println("</div>");
				body.println("<div style='margin-top: 5px; font-size:12px;'>");
				body.println("Features: PR: " + rank_page[node] + " TR: " + rank_thermo[node]);
				body.println("</div>");
				body.println("</div>");
			}
			
			idx = sortIndices(nodes, rank_thermo);
			body.println("</div><div style='float: right; width: 45%; margin-right: 10px;'>");
			for(int i = 0; i < limit; ++i)
			{
				int node = idx[i];
				
				body.println("<div style='overflow: hidden; margin: 0px auto; width:90%; margin-bottom: 20pt; border-color: grey; border-style: dashed; border-width: 1px; padding: 5px;'>");
				body.println("<div style='font-size:14px; '>");
				body.println("<a href='/" + node + "'>" + urls.get(node) + "</a>");
				body.println("</div>");
				body.println("<div style='margin-top: 5px; font-size:12px;'>");
				body.println("Features: PR: " + rank_page[node] + " TR: " + rank_thermo[node]);
				body.println("</div>");
				body.println("</div>");
			}
			body.println("</div>");
		}
		else
			body.println("<h1>Leere Menge</h1>");
		body.println(page_search_footer);
	}
	
	private void printStartPage(PrintStream body)
	{
		body.print(page_start);
	}
}
*/
