/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.tests;

import krake.Common;
import org.junit.Test;

import java.util.regex.Matcher;

import static org.junit.Assert.*;

/**
 * Tests for regexp-patterns in krake.Common.
 */
public class PatternTests
{
	@Test
	public void hrefLinkNoSpaces()
	{
		String test = "<a otherattribute=\"0\" href=\"http://somesite.com/sdfdfgdfg/dsfg dfg/bla\"> sdfsdg sdfg df";
		Matcher m = Common.LinkPattern.matcher(test);
		
		assertTrue(m.find());
		assertEquals(m.groupCount(), 2);
		
		String href = "";
		if(m.start(1) != -1 && m.end(1) != -1)
			href = test.substring(m.start(1), m.end(1)).trim();
		else if(m.start(2) != -1 && m.end(2) != -1)
			href = test.substring(m.start(2), m.end(2)).trim();
		assertEquals(href, "http://somesite.com/sdfdfgdfg/dsfg dfg/bla");
	}
	
	@Test
	public void hrefLinkSpaces()
	{
		String test = "<   a otherattribute=\"0\"   href  =   \"http://somesite.com/sdfdfgdfg/dsfg dfg/bla\"> sdfsdg sdfg df";
		Matcher m = Common.LinkPattern.matcher(test);
		
		assertTrue(m.find());
		assertEquals(m.groupCount(), 2);
		
		String href = "";
		if(m.start(1) != -1 && m.end(1) != -1)
			href = test.substring(m.start(1), m.end(1)).trim();
		else if(m.start(2) != -1 && m.end(2) != -1)
			href = test.substring(m.start(2), m.end(2)).trim();
		assertEquals(href, "http://somesite.com/sdfdfgdfg/dsfg dfg/bla");
	}
	
	@Test
	public void nonHref()
	{
		String test = "< sometag  otherattribute=\"0\"   href  =   \"http://somesite.com/sdfdfgdfg/dsfg dfg/bla\"  > sdfsdg sdfg df";
		String test2 = "< sometag  otherattribute=\"0\"   href  =   \"http://somesite.com/sdfdfgdfg/dsfg dfg/bla\"  > sdfsdg sdfg df";
		
		Matcher m = Common.LinkPattern.matcher(test);
		assertFalse(m.find());
		
		m = Common.LinkPattern.matcher(test2);
		assertFalse(m.find());
	}
	
	@Test
	public void frameAndiFrame()
	{
		String test = "<   frame otherattribute=\"0\"   src  =   \"http://somesite.com/sdfdfgdfg/dsfg dfg/bla\"> sdfsdg sdfg df";
		String test2 = "<   iframe otherattribute=\"0\"   src  =   \"http://somesite.com/sdfdfgdfg/dsfg dfg/bla\"> sdfsdg sdfg df";
		
		
		Matcher m = Common.FramePattern.matcher(test);
		assertTrue(m.find());
		assertEquals(m.groupCount(), 2);
		
		String href = "";
		if(m.start(1) != -1 && m.end(1) != -1)
			href = test.substring(m.start(1), m.end(1)).trim();
		else if(m.start(2) != -1 && m.end(2) != -1)
			href = test.substring(m.start(2), m.end(2)).trim();
		assertEquals(href, "http://somesite.com/sdfdfgdfg/dsfg dfg/bla");

		m = Common.FramePattern.matcher(test2);
		assertTrue(m.find());
		assertEquals(m.groupCount(), 2);
		
		href = "";
		if(m.start(1) != -1 && m.end(1) != -1)
			href = test2.substring(m.start(1), m.end(1)).trim();
		else if(m.start(2) != -1 && m.end(2) != -1)
			href = test2.substring(m.start(2), m.end(2)).trim();
		assertEquals(href, "http://somesite.com/sdfdfgdfg/dsfg dfg/bla");
	}
	
	@Test
	public void nonFrameAndiFrame()
	{
		String test = "<   img otherattribute=\"0\"   src  =   \"http://somesite.com/sdfdfgdfg/dsfg dfg/bla\"> sdfsdg sdfg df";
		String test2 = "<   iiframe otherattribute=\"0\"   src  =   \"http://somesite.com/sdfdfgdfg/dsfg dfg/bla\"> sdfsdg sdfg df";

		Matcher m = Common.FramePattern.matcher(test);
		assertFalse(m.find());
		
		m = Common.FramePattern.matcher(test2);
		assertFalse(m.find());
	}
}
