/*
Krake Webcrawlerframework
Copyright (C) LS2 TU Dortmund (http://ls2-www.cs.tu-dortmund.de)

This file is part of the Krake Webcrawlerframework.
The sourcecode and data files provided come without any warranty. Use at your own risk!

The sourcecode is the intellectual property of its authors. You have permission to download
and use this sourcecode for the sake of inspection and for private and academic use. You are
not allowed to redistribute this sourcecode without our permission. If you want to use the
sourcecode in any other way as granted above,
please contact Christian Sohler <christian.sohler[at]tu-dortmund.de>.
*/
package krake.tests;

import krake.KrakeConfig;
import krake.keys.KeyInfo;
import krake.keys.KeyStorage;
import krake.keys.KeyStorageMySQL;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.math.BigInteger;
import java.sql.SQLException;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

/**
 * Tests for krake.keys.KeyStorageMySQL.
 * Do NOT perform these tests with servers that contain valueable data, the tests
 * will delete and modify data!
 */
public class StorageMySQLTest
{

	@BeforeClass
	public static void setUpBeforeClass() throws Exception
	{
		// configparameters for a testing-server
		// adjust these as needed
		KrakeConfig.mysql_db = "krake";
		KrakeConfig.mysql_user = "root";
		KrakeConfig.mysql_passwd = "root";
		KrakeConfig.mysql_port = 9090;
		KrakeConfig.key_backend = "mysql";
		KrakeConfig.key_replication = 0;

		KrakeConfig.servers = new String[]{"localhost"};
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception
	{
		try
		{
			// at least try to cleanup
			KeyStorageMySQL ks = new KeyStorageMySQL(KrakeConfig.servers, KrakeConfig.mysql_db, KrakeConfig.mysql_user, KrakeConfig.mysql_passwd);
			ks.dropTables();
			ks.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.err.println("Cleanup failed, please drop the krake table manually!");
		}
	}

	@Test
	public void testKeyConversion()
	{
		BigInteger ikey = new BigInteger("DEADBABE", 16);
		String converted = ikey.toString(16);
		converted = converted.toUpperCase();
		assertEquals(converted, "DEADBABE");
	}

	@Test
	public void testConnectDisconnect() throws IOException
	{
		KeyStorageMySQL ks = new KeyStorageMySQL(KrakeConfig.servers, KrakeConfig.mysql_db, KrakeConfig.mysql_user, KrakeConfig.mysql_passwd);
		ks.close();
	}

	@Test
	public void testConnectDisconnectFromConfig() throws IOException
	{
		KeyStorage ks = KeyStorage.createFromConfig();
		ks.close();
	}

	@Test
	public void testDropCreateDrop() throws IOException, SQLException
	{
		KeyStorageMySQL ks = new KeyStorageMySQL(KrakeConfig.servers, KrakeConfig.mysql_db, KrakeConfig.mysql_user, KrakeConfig.mysql_passwd);
		ks.dropTables();
		ks.createTables();
		ks.dropTables();
	}

	@Test
	public void testCreateWriteReadDrop() throws IOException, SQLException
	{
		KeyStorageMySQL ks = new KeyStorageMySQL(KrakeConfig.servers, KrakeConfig.mysql_db, KrakeConfig.mysql_user, KrakeConfig.mysql_passwd);
		ks.createTables();

		ks.setKey("DEADBABE", KeyInfo.CRAWLED);
		ks.setKey("CAFE", KeyInfo.ERROR_GENERIC);

		assertTrue(ks.isKnown("DEADBABE"));
		assertTrue(ks.isKnown("CAFE"));
		assertTrue(!ks.isKnown("DEADCADE"));

		assertEquals(ks.getKey("DEADBABE"), KeyInfo.CRAWLED);
		assertEquals(ks.getKey("CAFE"), KeyInfo.ERROR_GENERIC);
		assertEquals(ks.getKey("DEADCADE"), null);

		ks.dropTables();
	}
}
